const path = require('path');
const express = require('express');
const webpack = require('webpack');
const webpackDevMiddleware = require('webpack-dev-middleware');
const webpackHotMiddleware = require('webpack-hot-middleware');
const app = express();
const testApiMiddleware = require('./server/test-api-middleware');

const webpackConfig = {
    // 默认模式
    mode: 'development',
    // 入口文件
    entry: {
        "test":path.resolve(__dirname, '../test/test.js'),
        "wdzxc":path.resolve(__dirname, '../src/index.js'),
    },
    // 输出配置
    output: {
        path: path.join(__dirname, '../static'),
        publicPath: "/static",                   // 生成文件的公共路径
        filename: "[name].js"
    },
    // 添加的module属性
    module: {
        rules: [
            // es6
            {
                test: /\.js$/,
                use: [{
                    loader: 'babel-loader',
                    options: {
                        cacheDirectory: true,
                        presets: "es2015"
                    }
                }]
            },
        ]
    },
    devServer:{
        historyApiFallback: {},
        noInfo: true
    },
    devtool:'#inline-source-map',
};

// webpack dev middleware & webpack-hot-middleware
var compiler = webpack(webpackConfig);
var devMiddleware = webpackDevMiddleware(compiler, {
    publicPath: webpackConfig.output.publicPath,
    quiet: true,
});
var hotMiddleware = webpackHotMiddleware(compiler, {
    log: () => {
    }
});
// force page reload when html-webpack-plugin template changes
compiler.plugin('compilation', function (compilation) {
    // compilation.plugin('html-webpack-plugin-after-emit', function (data, cb) {
    //     // hotMiddleware.publish({action: 'reload'});
    //     cb()
    // })
});
app.use(devMiddleware);
app.use(hotMiddleware);

// 静态文件
app.use(express.static('./build/public'));
// 测试接口
app.use(express.static('./build/public'));
app.use('/',testApiMiddleware);

console.log('> Starting dev server...');
devMiddleware.waitUntilValid(() => {
    console.log('> Listening at 3000\n');
});
app.listen(9102);

