/**
 * 对话框组件
 * @author wdzxc
 */

/*===============================================================================================@

 ## 功能说明

 ### $Dialog.notify
 * param1: string > 显示字符串
 * param2: duration > ms
 * param3: icon > 图标 1、√  2、× 3、★
 *
 ### $Dialog.alert
 * param1: string > 显示字符串
 * param2: callback > 点击确认后的回调

 ### $Dialog.confirm
 * param1: string|{tip,confirm,cancel,style,width} > 显示字符串
 * param2: callback > 点击确认后的回调
 * param3: errCallback > 点击取消后的回调

 ### $Dialog.registerModal
 * param :
 > [ xxModal:{ } ]
 #### xxModal
 ```
 $Dialog.registerModal({
 submitModal:{
 target:'#submitModal',
 transMtd:'Z-modal_show'
 }
 });
 ```
 #### $Dialog.xxModal( 'show'| 'hide' )


 @==============================================================================================*/

// 外部依赖
import $wd from '../../utils';

// 添加CSS样式
// require("./assets/css/dialog.scss");
const style = `
    <style type="text/css">
        .wd-modal {
            position: fixed;
            z-index: 1050;
            left: 0;
            top: 0;
            right: 0;
            bottom: 0;
            padding: 0;
            margin: 0;
            border: 0;
            -webkit-box-sizing: border-box;
            box-sizing: border-box;
            overflow: auto;
            background: rgba(0, 0, 0, 0.5);
            color: #48576a;
            text-align: center;
            vertical-align: sub;
            opacity: 0;
            -webkit-transition-duration: 300ms;
            -o-transition-duration: 300ms;
            transition-duration: 300ms;
        }
        .wd-modal.active {
            opacity: 1;
        }
        .wd-modal:after {
            content: "";
            display: inline-block;
            height: 100%;
            width: 0;
            vertical-align: middle;
        }
        .wd-modal .confirm {
            display: inline-block;
            overflow: auto;
            -webkit-border-radius: 6px;
            border-radius: 6px;
            width: 270px;
            -webkit-background-clip: padding-box;
            background-clip: padding-box;
            background-color: rgba(253, 253, 253, 0.95);
            position: relative;
            margin: 0 auto;
        }
        .wd-modal .confirm .confirm-tip {
            min-height: 42px;
            -webkit-border-top-left-radius: 6px;
            border-top-left-radius: 6px;
            -webkit-border-top-right-radius: 6px;
            border-top-right-radius: 6px;
            padding: 18px;
            font-size: 16px;
            display: -webkit-box;
            display: -webkit-flex;
            display: -ms-flexbox;
            display: flex;
            word-break: break-all;
            word-break: break-word;
            max-height: 80vh;
            overflow-y: auto;
        }
        .wd-modal .confirm .confirm-tip div {
            width: 100%;
        }
        .wd-modal .confirm .confirm-tip .confirm-content {
            font-size: 16px;
            width: 100%;
            text-align: center;
        }
        .wd-modal .confirm .confirm-ft {
            display: -webkit-box;
            display: -webkit-flex;
            display: -ms-flexbox;
            display: flex;
            width: 100%;
            -webkit-border-bottom-left-radius: 6px;
            border-bottom-left-radius: 6px;
            -webkit-border-bottom-right-radius: 6px;
            border-bottom-right-radius: 6px;
            -webkit-box-sizing: border-box;
            box-sizing: border-box;
            overflow: hidden;
        }
        .wd-modal .confirm .confirm-ft button {
            display: block;
            -webkit-box-flex: 1;
            -webkit-flex: 1;
            -ms-flex: 1;
            flex: 1;
            width: 100%;
            height: 42px;
            line-height: 42px;
            background: transparent;
            text-align: center;
            font-size: 16px;
            border: 1px solid #c8c7cc;
            border-right-width: 0;
            -webkit-box-sizing: border-box;
            box-sizing: border-box;
            cursor: pointer;
        }
        .wd-modal .confirm .confirm-ft button:first-child {
            border-left-width: 0;
        }
        .wd-modal .confirm .confirm-ft .selected {
            color: #00a5e0;
        }
    </style>
`;
// let messageBoxAlertTemplate = require("./assets/views/message-box-alert.html");
let messageBoxAlertTemplate = `
    <div class="wd-modal">
        <div class="confirm">
            <div class="confirm-tip">
                <div class="confirm-content"></div>
            </div>
            <div class="confirm-ft">
                <button type="button" data-name="confirm" data-role="button" id="dialogButton1" class="selected">确认</button>
            </div>
        </div>
    </div>
`;
// let messageBoxConfirmTemplate = require("./assets/views/message-box-confirm.html");
let messageBoxConfirmTemplate = `
    <div class="wd-modal">
        <div class="confirm">
            <div class="confirm-tip">
                <div class="confirm-content"></div>
            </div>
            <div class="confirm-ft">
                <button type="button" data-name="cancel" data-role="button" id="dialogButton0">取消</button>
                <button type="button" data-name="confirm" data-role="button" id="dialogButton1" class="selected">确认</button>
            </div>
        </div>
    </div>
`;


// HTML 模板
let htmlTemplate = {
    createElemByHtml: function (str) {
        let tempDiv = document.createElement('div');
        tempDiv.innerHTML = str;
        if (tempDiv.childElementCount <= 0) {
            return null;
        } else if (tempDiv.childElementCount == 1) {
            return tempDiv.children[0];
        }
        return tempDiv.children;
    },
    getAlertElem: function () {
        return this.createElemByHtml(messageBoxAlertTemplate);
    },
    getConfirmElem: function () {
        return this.createElemByHtml(messageBoxConfirmTemplate);
    }
};
document.head.appendChild(htmlTemplate.createElemByHtml(style));

/**
 * 警告对话框
 * <button class="btn btn-success btn-sm" onclick="$Dialog.alert('我是一个警告对话框！~\(≧▽≦)/~')">点击试下</button>
 * @param {string|{tip,confirm}} str 提示信息，字符串类型为提示文本，对象类型如下
 * @param {string|{tip,confirm}} str.tip 提示文本
 * @param {string|{tip,confirm}} str.confirm 确认按钮文本
 * @param {function=} callback 点击确认后的回调函数
 * @memberof module:DialogBox
 */
function alert(str, callback) {
    let alertElem = htmlTemplate.getAlertElem();
    if (typeof str == "string") {
        alertElem.querySelector('.confirm-content').innerHTML = str;
    }
    if (typeof str == "object") {
        alertElem.querySelector('.confirm-content').innerHTML = str.tip;
        alertElem.querySelector('[data-name="confirm"]').innerHTML = str.confirm;
    }

    alertElem.style.display = "block";
    document.body.appendChild(alertElem);
    setTimeout(function () {
        $wd(alertElem).addClass("active");
    },1);

    alertElem.querySelector('.selected').onclick = function () {
        // 确认
        callback && callback();
        $wd(alertElem).removeClass("active");
        setTimeout(function () {
            alertElem.parentNode.removeChild(alertElem);
        },300);
    }
}

/**
 * 确认对话框
 * <button class="btn btn-success btn-sm" onclick="$Dialog.confirm('我是一个确认对话框！~\(≧▽≦)/~')">点击试下</button>
 * @param {string|{tip,confirm,cancel,style,width}} str 提示信息，字符串类型为提示文本，对象类型如下
 * @param {string} [str.tip="提示信息"] 提示文本
 * @param {string} [str.confirm="确定"]确认按钮文本
 * @param {string} [str.cancel="取消"] 取消按钮文本
 * @param {string} [str.style=""] 附加样式
 * @param {function=} confirmCallback 点击确认后回调函数
 * @param {function=} cancelCallback 点击取消后回调函数
 * @memberof module:DialogBox
 */
function confirm(str, confirmCallback, cancelCallback) {
    let alertElem = htmlTemplate.getConfirmElem();
    let textElem = alertElem.querySelector('.confirm-content');
    alertElem.querySelector('.confirm-content').innerHTML = str;
    if (typeof str == "string") {
        alertElem.querySelector('.confirm-content').innerHTML = str;
    }
    if (typeof str == "object") {
        textElem.innerHTML = str.tip;
        alertElem.querySelector('[data-name="confirm"]').innerHTML = str.confirm ? str.confirm : "确认";
        alertElem.querySelector('[data-name="cancel"]').innerHTML = str.cancel ? str.cancel : "取消";
        textElem.setAttribute('style', (textElem.getAttribute('style') || "") + (str.style || ""));
        if (str.width) {
            alertElem.querySelector('.confirm').style.width = str.width;
        }
    }
    alertElem.style.display = "block";
    document.body.appendChild(alertElem);

    // $wd(alertElem).addClass("active");
    setTimeout(function () {
        $wd(alertElem).addClass("active");
    },1);

    function hide() {
        $wd(alertElem).removeClass("active");
        setTimeout(function () {
            alertElem.parentNode.removeChild(alertElem);
        },300);
    }

    alertElem.querySelector('#dialogButton0').onclick = function () {
        // 取消
        cancelCallback && cancelCallback();
        hide();
    };

    alertElem.querySelector('#dialogButton1').onclick = function () {
        // 确认
        confirmCallback && confirmCallback();
        hide();
    };
    return textElem;
}

export {
    alert,confirm
};