/**
 * 公共函数库
 * @author wdzxc
 * @version 0.0.22
 */

import {
    isUndefined,
    isObject,
    isArray,
    isFunction,
    isString,
    isNumber,
    isElement,
    isElementArray,
    isTableElement,
    isWhat
} from "./isWhat";
import {objClone, getObjAttr, setObjAttr, objCover, objClear, objEqual, objExtract, objectMix} from "./objectOpt";
import {
    $wd,
    $wds,
    showElem,
    hideElem,
    insertAfter,
    insertAfterTable,
    createElementByHtml,
    createElemByTemplate,
    getFormData
} from "./dom";
import {parseURL, buildURL} from "./url";

require("./src/array-polyfill");

//=========================================================
//  兼容性处理
//=========================================================
// console
console.log || (console.log = new Function());
console.warn || (console.warn = console.log);
console.error || (console.error = console.log);
if (typeof Symbol === 'undefined') {
    var Symbol = function () {
        return {};
    };
}

//=========================================================
// 数组操作
//=========================================================

/**
 * 数组遍历
 * @param array 数组
 * @param callback 回调函数 value,index,array
 * @memberof $wd
 */
function forEach(array, callback) {
    [].forEach.call(array, callback);
}

/**
 * 检查数组内是否有目标数据项 简单遍历检查
 * @param array 数组
 * @param targetData 匹配数值
 * @returns {boolean}
 * @memberof $wd
 */
function arrayHas(array, targetData) {
    let isMatch = false;
    [].forEach.call(array, function (data) {
        if (targetData == data) {
            isMatch = true;
        }
    });
    return isMatch;
}

/**
 * 移除数组内与目标数据相同项 会修改原数组
 * @param array 数组
 * @param targetData 目标数据
 * @returns {*}
 * @memberof $wd
 */
function arrayRemove(array, targetData) {
    let newArray = [];
    [].forEach.call(array, function (data) {
        if (data !== targetData) newArray.push(data);
    });
    [].splice.apply(array, [0, array.length].concat(newArray));
    return array;
}

//=========================================================
// 链式调用 Promise
// $wd.promise( function(next){ next(data) }).then( function(next,data){ next() })
// 如果使用 .then.then.sync() 会同步开始调用过程
//=========================================================
/**
 * 链式调用&Promise
 * @param callback {function} 回调函数
 * @returns object
 * @memberof $wd
 * @example
 *
 * $wd.promise(function (next) {
 *    console.log('1');
 *    next();
 * })
 * .then(function (next) {
 *    console.log('2');
 *    next();
 * })
 * .then(function (next) {
 *    console.log('3');
 *    next();
 * }).sync();
 * console.log("start");
 *
 * // => 1 2 3 start
 * // 去掉 .sync()
 * // => start 1 2 3
 *
 */
function promise(callback) {
    let start = false, resolveQueue = [], rejectQueue = [], resolve, reject;
    resolve = function () {
        rejectQueue.shift();
        (resolveQueue.shift() || function () {
        }).apply(this, [resolve, reject].concat([].slice.call(arguments, 0)));
    };
    reject = function () {
        (rejectQueue.shift() || function () {
        }).apply(this, [].concat([].slice.call(arguments, 0)));
        rejectQueue = [];
    };
    setTimeout(function () {
        !start && callback(resolve, reject);
    }, 1);
    return {
        then: function (resolve, reject) {
            resolveQueue.push(resolve);
            rejectQueue.push(reject);
            return this;
        },
        sync: function () {
            start = true;
            callback(resolve, reject);
        }
    };
}

/**
 * 检查为空的数据项
 * @param {object|array}  obj 检查目标,对象 对象数组
 * @param callback 检查到空值触发回调 参数 value name
 * @param {(array|string)=} exception 例外的属性名称列表
 * @returns {boolean} 如果有空值返回false
 * @memberof $wd
 * @example
 *
 * // 示例
 * let obj = {a:1,b:'',c:''};
 * console.log( $wd.checkEmpty(obj, function (name) {
     *     console.log(name);
     * }) );
 * // => b,false
 * console.log( $wd.checkEmpty(obj, function (name) {
     *     console.log(name);
     * }),'b');
 * // => c,false
 * console.log( $wd.checkEmpty(obj, function (name) {
     *     console.log(name);
     * }),'b,c');
 * // => true
 *
 */
function checkEmpty(obj, callback, exception) {
    let originObj = obj, ignoreList = [];
    if (isObject(obj)) {
        originObj = [];
        originObj.push(obj);
    }
    exception && (ignoreList = isArray(exception) ? exception : exception.split(','));
    let isMatch = false;
    originObj.forEach(function (data) {
        Object.keys(data).forEach(function (key) {
            let value = data[key];
            if (arrayHas(ignoreList, key) || isMatch) return;
            if ((value == '' && value !== 0 && value !== false) || isUndefined(value)) {
                callback(key, data);
                isMatch = true;
            }
        });
    });
    return !isMatch;
}


//=========================================================
//  异步加载资源
//=========================================================
/**
 * 获取JS
 * @param {string} src JS路径
 * @param {function} callback 获取成功后回调
 * @returns {boolean} 返回true表示页面上没有这个JS开始进行加载，返回false表示已经存在这个JS文件不进行加载
 * @memberof $wd
 */
function getScript(src, callback) {
    // 页面脚本
    if (src) {
        let curScripts = document.querySelectorAll('script');
        for (let i = 0; i < curScripts.length; i++) {
            if (curScripts[i].getAttribute('src') == src) {
                callback('exist');
                return false;
            }
        }
        let pageScript = document.createElement('script');
        pageScript.type = 'text/javascript';
        pageScript.src = src;
        document.head.appendChild(pageScript);
        pageScript.onload = function () {
            console.log("script load success [" + this.getAttribute('src') + "]");
            callback(true);
        };
        pageScript.onerror = function () {
            console.log("script load error [" + this.getAttribute('src') + "]");
            callback('error');
        };
    }
}

//=========================================================
//  时间
//=========================================================
/**
 * 时间格式化
 * @param {string} fmt 时间格式化字符串
 * @example
 * y : year
 * M : month
 * h : hour
 * m : minute
 * s : second
 * q : quarter
 * S : millisecond
 */
isUndefined(Array.prototype.format) && (Date.prototype.format = function (fmt) {
    let o = {
        "M+": this.getMonth() + 1,  // 月份
        "d+": this.getDate(),       // 日
        "h+": this.getHours(),      // 小时
        "m+": this.getMinutes(),    // 分
        "s+": this.getSeconds(),    // 秒
        "q+": Math.floor((this.getMonth() + 3) / 3),    // 季度
        "S": this.getMilliseconds()                     // 毫秒
    };
    if (/(y+)/.test(fmt)) fmt = fmt.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
    for (let k in o)
        if (new RegExp("(" + k + ")").test(fmt))
            fmt = fmt.replace(RegExp.$1, (RegExp.$1.length == 1) ?
                (o[k]) :
                (("00" + o[k]).substr(("" + o[k]).length)));
    return fmt;
});

/**
 * 时间间隔
 * @param targetDate
 * @param cmpDate
 * @memberof $wd
 * @returns {{gapSeconds: Number, gapMinutes: Number, gapHours: Number, gapDate: Number}}
 * @memberof $wd
 */
function getDateInterval(targetDate, cmpDate) {
    let gap = (targetDate - (cmpDate ? cmpDate : new Date())) / 1000;
    return {
        gapSeconds: parseInt(gap % 60),
        gapMinutes: parseInt(gap / (60)) % 60,
        gapHours: parseInt(gap / (60 * 60)) % 24,
        gapDate: parseInt(gap / (60 * 60 * 24))
    }
}

/**
 * 集中调度封装
 * 封装函数的调用过程，置入钩子函数来过滤参数、控制执行过程
 * @param {object} obj  需要封装调用的方法集合
 * @param {function=} hookFunc 置入调用过程的钩子函数 第一个参数为函数自身 第二个参数为函数调用参数
 * @memberof $wd
 */
function invokeProxy(obj, hookFunc) {
    let callback = $wd.isFunction(hookFunc) ? hookFunc : function () {
        return true;
    };
    for (let an in obj) {
        if (!obj.hasOwnProperty(an)) continue;
        if ($wd.isFunction(obj[an])) {
            !function () {
                let func = obj[an];
                obj[an] = function () {
                    return callback(func, arguments) && func.apply(this, arguments);
                };
            }();
        }
    }
    return obj;
}

//=========================================================
//  对外提供的功能
//=========================================================
// $wd
$wd.objClone = objClone;
$wd.getObjAttr = getObjAttr;
$wd.createElementByHtml = createElementByHtml;
$wd.castElemByTemplate = createElemByTemplate;
$wd.createElemByTemplate = createElemByTemplate;

$wd.parseURL = parseURL;
$wd.buildURL = buildURL;
$wd.showElem = showElem;
$wd.hideElem = hideElem;

$wd.forEach = forEach;
$wd.arrayHas = arrayHas;
$wd.arrayRemove = arrayRemove;
$wd.getFormData = getFormData;
$wd.insertAfter = insertAfter;
$wd.insertAfterTable = insertAfterTable;

$wd.objCover = objCover;
$wd.objClear = objClear;
$wd.objEqual = objEqual;
$wd.objExtract = objExtract;
$wd.objectMix = objectMix;
$wd.setObjAttr = setObjAttr;
$wd.checkEmpty = checkEmpty;

$wd.promise = promise;
$wd.getScript = getScript;
$wd.invokeProxy = invokeProxy;
$wd.getDateInterval = getDateInterval;

// export
$wd.$wd = $wd;
$wd.$wds = $wds;
$wd.isArray = isArray;
$wd.isNumber = isNumber;
$wd.isObject = isObject;
$wd.isString = isString;
$wd.isElement = isElement;
$wd.isFunction = isFunction;
$wd.isUndefined = isUndefined;
$wd.isElementArray = isElementArray;
$wd.isTableElement = isTableElement;

$wd.isWhat = isWhat;

// module.exports = $wd;
export default $wd;
